#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <comedi.h>
#include <getopt.h>
#include <errno.h>

#define COMEDI_VERSION "0.4.0"

int quiet=0,verbose=0;

void do_help(int i)
{
	fputs(
"comedi_config version " COMEDI_VERSION "\n"
"usage:\n"
"  comedi_config [-[vVq]] <device file> <driver> [<driver parameters...>]\n"
"where <driver parameters> are one of\n"
"  port 0x###   -- I/O port base address\n"
"  irq #        -- IRQ number\n"
"  irq probe    -- automatically probe for IRQ\n"
"  no_irq       -- specify not to use interrupts\n"
"The defaults are to try to probe for port/irq, which will typically\n"
"fail for probing I/O port and is a bad idea anyway.\n",stderr);
	exit(i);
}

int main(int argc,char *argv[])
{
	comedi_devconfig it;
	int fd;
	int port,irq,c;
	char *fn,*type;
	
	while(1){
		c=getopt(argc,argv,"vVq");
		if(c==-1)break;
		switch(c){
		case 'v':
			verbose=1;
			break;
		case 'V':
			fputs("comedi_config version " COMEDI_VERSION "\n",stderr);
			exit(0);
			break;
		case 'q':
			quiet=1;
			break;
		default:
			do_help(1);
		}
	}
	port=-1;
	irq=-1;
	if(optind>=argc)do_help(0);
	fn=argv[optind];
	optind++;
	if(optind>=argc){
		puts("Info not available");
		exit(0);
	}
	type=argv[optind];
	optind++;
	while(optind<argc){
		if(!strcmp(argv[optind],"port")){
			optind++;
			if(optind>=argc)do_help(1);
			if(!sscanf(argv[optind],"%i",&port)){
				fprintf(stderr,"Can't read port \"%s\"\n",
					argv[optind]);
				do_help(1);
			}
			optind++;
		}else if(!strcmp(argv[optind],"irq")){
			optind++;
			if(optind>=argc){
				fputs("Missing irq number\n",stderr);
				do_help(1);
			}
			if(!strcmp(argv[optind],"probe")){
				irq=-1;
			}else if(!sscanf(argv[optind],"%i",&irq)){
				fprintf(stderr,"Can't read irq \"%s\"\n",
					argv[optind]);
				do_help(1);
			}
			optind++;
		}else if(!strcmp(argv[optind],"no_irq")){
			optind++;
			irq=0;
		}else do_help(1);
	}

	strcpy(it.name,type); /* fix: strncpy */
	it.iobase=port;
	it.irq=irq;

	fd=open(fn,O_RDWR);
	if(fd<0){
		fprintf(stderr,"Can't open device %s\n",fn);
		exit(1);
	}
	/* add: sanity check for device */
	
if(verbose){
	printf("configuring type=%s port=0x%04x irq=%d\n",
		it.name,it.iobase,it.irq);
}
	if(ioctl(fd,COMEDI_CONFIG,&it)<0){
		fprintf(stderr,"Configure failed! errno=%d\n",errno);
		exit(1);
	}
	exit(0);
}

