/*
    guile-comedi/gc.c
    wrapper functions for comedilib

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include <stdio.h>
#include <math.h>
#include <guile/gh.h>
#include "guile-comedi.h"

SCM gc_test(SCM s_n)
{
	return s_n;
}

/* this is a factorial routine in C, made to be callable by Scheme */
SCM c_factorial(SCM s_n)
{
  int i;
  unsigned long result = 1, n;

  n = gh_scm2ulong(s_n);

  gh_defer_ints();
  for (i = 1; i <= n; ++i) {
    result = result*i;
  }
  gh_allow_ints();
  return gh_ulong2scm(result);
}

/* a sin routine in C, callable from Scheme.  it is named c_sin() to
   distinguish it from the default Scheme sin function */
SCM c_sin(SCM s_x)
{
  double x = gh_scm2double(s_x);

  return gh_double2scm(sin(x));
}

/* play around with vectors in Guile: this routine creates a vector of
   the given length, initializes it all to zero except element 2 which
   is set to 1.9.  */
SCM vector_test(SCM s_length)
{
  SCM xvec;
  unsigned long c_length;

  c_length = gh_scm2ulong(s_length);
  printf("requested length for vector: %ld\n", c_length);

  /* create a vector */
  xvec = gh_vector(c_length, gh_double2scm(0.0));
  /* set the second element in it */
  gh_vset(xvec, gh_int2scm(2), gh_double2scm(1.9));

  return xvec;
}

