/*
    include/comedi.h (installed as /usr/include/comedi.h)
    header file for comedi

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef _COMEDI_H
#define _COMEDI_H

#define COMEDI_TRIGNOW 0
#define COMEDI_TRIGCLK 1
#define COMEDI_TRIGEXT 2
#define COMEDI_TRIG100HZ 3

#define COMEDI_RDONLY 1
#define COMEDI_RDWR 0
#define COMEDI_SHARED 2


#define COMEDI_IO 0
 #define COMEDI_IO_INPUT 0
 #define COMEDI_IO_OUTPUT 1
 #define COMEDI_IO_BIDIR 2
#define COMEDI_AD 1
 #define COMEDI_AD_ANALOG 1
 #define COMEDI_AD_DIGITAL 0
#define COMEDI_BITS 2
#define COMEDI_MAXDATA 3
#define COMEDI_RANGE 4 /* integer describing a range selection -- user
	must look up actual range in a library */
#define COMEDI_LASTDATA 5  /* last data sample */
#define COMEDI_LASTSEC 6  /* time of last data sample (seconds) */
#define COMEDI_LASTNSEC 7  /* time of last data sample (nanoseconds) */
#define COMEDI_AREF 8  /* analog reference mode */
 #define COMEDI_AREF_GND 0  /* referenced to analog ground */
 #define COMEDI_AREF_COM 1  /* referenced to floating common */
 #define COMEDI_AREF_DIFF 2  /* referenced to another channel */
			     /* i.e., differential input */
 #define COMEDI_AREF_OTHER 3  /* some other reference */
#define COMEDI_DITHER 9  /* dithering, if available */


#define COMEDI_DEGLITCH 32  /* true=1 */

#define COMEDI_STROBE 48  /* strobed output true=1 */
#define COMEDI_IOBITGROUP 49  /* how many bits are grouped together for
				i/o designation */
#define COMEDI_IOBITS 50  /* bitfield describing i/o of each signal line */
 #define COMEDI_IOBITS_INPUT 0
 #define COMEDI_IOBITS_OUTPUT 0xffffffff

#define COMEDI_COUNT 64  /* count number for counters & reset */


typedef struct{
	unsigned int chan;
	int io;
	unsigned int n;
	int trigsrc;
	int trigvar;
	unsigned int data;
	unsigned int job;
}comedi_trig;

typedef struct{
	unsigned int job;
	unsigned int chan;
	unsigned int data;
}comedi_sample;

typedef struct{
	char name[20];
	int iobase;
	int irq;
	int dma;
	int j_ai_differential;
	int j_ai_twoscomp;
	int j_ao0_range;
	int j_ao1_range;
	int j_ao_twoscomp;
}comedi_devconfig;

typedef struct{
	unsigned int chan;
	unsigned int flags;
	unsigned int pnum;
	int pval;
}comedi_param;

typedef struct{
	char driver[20];
	char board[20];
	int iobase;
	int iosize;
	int irq;
	int minor;
	int chan;
}comedi_info;

typedef struct{
	void *local;
	unsigned int remote;
	unsigned int nbytes;
}comedi_memcpy;

/* ioctls */

#define CIO 'd'
#define COMEDI_CONFIG _IOW(CIO,0,comedi_devconfig)
#define COMEDI_GETINFO _IOR(CIO,1,comedi_info)
#define COMEDI_TRIG _IOWR(CIO,2,comedi_trig)
#define COMEDI_GETPARAM _IOWR(CIO,17,comedi_param)
#define COMEDI_SETPARAM _IOW(CIO,18,comedi_param)
#define COMEDI_GETNPARAM _IOWR(CIO,19,comedi_param)

#define COMEDI_COPY_FROM _IOW(CIO,32,comedi_memcpy)
#define COMEDI_COPY_TO _IOW(CIO,33,comedi_memcpy)
#define COMEDI_MEMCPY _IOW(CIO,34,comedi_memcpy)
#define COMEDI_RESET _IO(CIO,35)
#define COMEDI_EXEC _IO(CIO,36)

#endif /* _COMEDI_H */

