/*
    module/comedi_module.h
    header file for kernel-only structures, variables, and constants

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _COMEDI_MODULE_H
#define _COMEDI_MODULE_H


#if LINUX_VERSION_CODE < 2*65536+1*256+0
#define OLD_KERNEL 1
#else
#define OLD_KERNEL 0
#endif

#include <comedi.h>

#define COMEDI_VERSION "0.4.2"

#define COMEDI_MAJOR  98
#define COMEDI_DEVICES 16


typedef struct comedi_device_struct comedi_device;

typedef int comedi_devinit(comedi_device *,comedi_devconfig *);

typedef struct comedi_bufll{
	struct comedi_bufll *next;
	int use_count;
	char stuff[0];
}comedi_bufll;

typedef struct{
	comedi_bufll *first;
	comedi_bufll *last;
	int in;
	int use_count;
	int num_pages;
}comedi_buf;

typedef struct{
	comedi_buf *buf;
	comedi_bufll *bufll;
	int offset;
}comedi_bufptr;

typedef struct comedi_iparam{
	struct comedi_iparam *next;
	int flags;
	int pnum;
	int pval;
}comedi_iparam;

typedef struct{
	comedi_iparam *paramlist;
	int (*trig)(comedi_device *,comedi_trig *);
	int (*sp)(comedi_device *,comedi_param *);
	comedi_iparam *lastdata,*lastsec,*lastnsec; /* shortcuts */
}comedi_chinfo;

struct comedi_device_struct{
	int use_count;
	void *private;
	int minor;
	int iobase;
	int iosize;
	int irq;
	int chan;
	char board[20];
	comedi_chinfo *chinfo;
	comedi_buf rdbuf;
	struct wait_queue *rdwait;
	int (*rem)(comedi_device *);
	void (*timer)(comedi_device *);
	int (*write)(comedi_device *,const char *src,int nbytes);
#if DSP
	int (*memcpy_to)(comedi_device *,int dest,void *src,int nbytes);
	int (*memcpy_from)(comedi_device *,void *dest,int src,int nbytes);
	int (*d_memset)(comedi_device *,int dest,int nbytes,int val);
	int (*memexec)(comedi_device *,int entry_pc);
	int (*reset)(comedi_device *,int entry_pc);
#endif
};

extern comedi_device *comedi_devices;
extern comedi_devinit *comedi_devinits[];
extern const int comedi_maxinits;

/*
 * function prototypes
 */

int comedi_initbuf(comedi_buf *buf);
void comedi_freebuf(comedi_buf *buf);
void copy_to_buffer(comedi_buf *buf,void *src,int n);
int copy_from_buffer(comedi_bufptr *ptr,void *dest,int n);
comedi_bufptr *comedi_hookbuf(comedi_buf *buf);
void comedi_unhookbuf(comedi_bufptr *it);
void report_sample(comedi_device *dev,comedi_sample *s);

int init_param(void);
int cleanup_param(void);
int addparam(comedi_iparam **list,int pnum,int flags,int pval);
int changeparam(comedi_iparam **list,int pnum,int flags,int pval);
int delparamlist(comedi_iparam *list);
int getparamval(comedi_iparam *list,int pnum);

#endif /* _COMEDI_MODULE_H */

