/*
 * Driver for parallel port
 * David A. Schleef
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>


#define PARPORT_SIZE 3

typedef struct{
	int cr;
}parport_private;
#define devpriv ((parport_private *)dev->private)

comedi_devinit parport_init;
static int parport_dio(comedi_device *dev,comedi_trig *ad);
static int parport_rem(comedi_device *dev);

static int parport_dio(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	switch(it->io){
	case COMEDI_IO_INPUT:
		s.data=it->data=inb(dev->iobase+PARPORT_DATA+it->chan);
		s.chan=it->chan;
	
		report_sample(dev,&s);
	
		return 0;
	case COMEDI_IO_OUTPUT:
		outb(it->data,dev->iobase+PARPORT_DATA+it->chan);
		return 0;
	default:
		return -EINVAL;
	}
}

static int parport_sp(comedi_device *dev,comedi_param *it)
{
	return -EINVAL;
}

int parport_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;

	if(strcmp("parport",it->name))
		return 0;

	printk("comedi%d: parport: 0x%04x ",dev->minor,it->iobase);
	if(check_region(it->iobase,PARPORT_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(it->iobase,PARPORT_SIZE,"parport (comedi)");
	dev->iobase=it->iobase;
	dev->iosize=PARPORT_SIZE;
	dev->irq=0;

	dev->chan=3;

	devpriv=kmalloc(sizeof(parport_private)+dev->chan*sizeof(comedi_chinfo),
		GFP_KERNEL);
	if(!devpriv)return -ENOMEM;
	memset(devpriv,0,sizeof(parport_private)+dev->chan*sizeof(comedi_chinfo));

	dev->chinfo=(void *)(devpriv+1);
	for(i=0;i<3;i++){
		dev->chinfo[i].trig=parport_dio;
		dev->chinfo[i].sp=parport_sp;
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_BIDIR);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITGROUP,COMEDI_RDONLY,(i==2)?4:8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITS,COMEDI_RDONLY,0xff&COMEDI_IOBITS_INPUT);
	}

	comedi_initbuf(&dev->rdbuf);

	dev->rem=parport_rem;

	printk("\n");
	return 1;
}


static int parport_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: parport: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(devpriv);
	release_region(dev->iobase,dev->iosize);

	comedi_freebuf(&dev->rdbuf);

	return 0;
}

