
/*

  this file needs to be compiled with the -O2 flag, otherwise outw()
  macros aren't expanded correctly.

  use:

  ni_init(int i)
    tells the driver where to find a board that already has a correctly
    running comedi device.  i is the I/O base address for the board.

  ni_ai(channel, analog_reference, gain, dither)
    gets one sample from specified channel.  Analog_reference, gain, and
    dither have the same meaning as with comedi.  These do not affect
    or depend on the values in the comedi device.
    returns sample data.
	
  ni_ao(channel, data)
    outputs one sample to specified channel.  Set the output range
    using the comedi device.  Note that depending on the output range,
    the samples may need a bit to be flipped--i.e., if the output is
    wrong, edit the source below.
  
  ni_di(), ni_do(data)
    digital input/ouput.  Set the I/O bits using the comedi device.

*/


#include <linux/ioport.h>
#include <linux/errno.h>
#include "ni_stc.h"
#include <linux/kernel.h>
#include <linux/delay.h>
#include <asm/io.h>

#undef win_out
#define win_out(a,b) _win_out((a),(b),iobase)

#undef win_in
#define win_in(b) _win_in((b),iobase)

static int ni_modebits1[4]={ 0x3000, 0x2000, 0x1000, 0 };
static int ni_modebits2[4]={ 0xf, 0xf, 0x7, 0x7 };

static ni_gainlkup[]={ 1, 2, 4, 7, 9, 10, 12, 15 };

/* Additional windowed registers unique to E series */

#define Configuration_Memory_Clear	82
#define ADC_FIFO_Clear			83

/* i/o port offsets */

/* 8 bit registers */
#define Status				0x01
#define Serial_Command			0x0d
#define Misc_Command			0x0f
#define Port_A				0x19
#define Port_B				0x1b
#define Port_C				0x1d
#define Configuration			0x1f
#define Strobes				0x01
#define Channel_A_Mode			0x03
#define Channel_B_Mode			0x05
#define Channel_C_Mode			0x07
#define AI_AO_Select			0x09
#define G0_G1_Select			0x0b

/* 16 bit registers */
#define Configuration_Memory_Low	0x10
#define Configuration_Memory_High	0x12
#define ADC_FIFO_Data_Register		0x1c
#define AO_Configuration		0x16
#define DAC_FIFO_Data			0x1e
#define DAC0_Direct_Data		0x18
#define DAC1_Direct_Data		0x1a


static int iobase;


int ni_ai(int chan,int gain,int aref,int dither)
{
	int ch,aip;
	int data;
	int i,to;
	
	win_out(1,ADC_FIFO_Clear);
	win_out(1,Configuration_Memory_Clear);

	ch=ni_modebits1[aref]|(chan&ni_modebits2[aref]);
	outw(0x1000|chan,iobase+Configuration_Memory_High);
	
	aip=((gain&0x8)<<5)|(gain&0x7)|(dither?0x200:0);
	outw(0x8000|aip,iobase+Configuration_Memory_Low);
	
	win_out(1,AI_Command_1_Register);
	for(i=0;i<1000;i++){
		if(!(inw(AI_Status_1+iobase)&
		  AI_FIFO_Empty_St)){
			win_out(1,ADC_FIFO_Clear);
			break;
		}
	}

	win_out(1,AI_Command_1_Register);
	
	for(i=0;i<1000;i++){
		if(!(inw(AI_Status_1+iobase)&
		  AI_FIFO_Empty_St)){
			data=inw(iobase+ADC_FIFO_Data_Register);
			if(!(aip&0x100))data^=0x800;
			data&=0xfff;
			return data;
		}
	}
	return -ETIME;
}

int ni_ao(int chan,int data)
{
	unsigned int i=data;
	
	if(i>=0x1000)return -EINVAL;
	if(chan==0){
		/*i^=0x800;*/  /* uncomment this to use other range */
		outw(i,iobase+DAC0_Direct_Data);
	}else{
		/*i^=0x800;*/
		outw(i,iobase+DAC1_Direct_Data);
	}
	return 0;
}

int ni_di(void)
{
	return win_in(DIO_Input_Register)&0xff;
}

int ni_do(int data)
{
	win_out(data&0xff,DIO_Output_Register);
}

int ni_init(int i)
{
	iobase=i;
}

